# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['poetry_plugin_sort']

package_data = \
{'': ['*']}

install_requires = \
['poetry>=1.2.0,<2.0.0']

entry_points = \
{'poetry.application.plugin': ['demo = '
                               'poetry_plugin_sort.plugins:SortDependenciesPlugin']}

setup_kwargs = {
    'name': 'poetry-plugin-sort',
    'version': '0.1.0b2',
    'description': 'Poetry plugin to sort the dependencies alphabetically',
    'long_description': '# Poetry Plugin: Dependencies sorting\n\n[![PyPI Version](https://img.shields.io/pypi/v/poetry-plugin-sort?label=PyPI)](https://pypi.org/project/poetry-plugin-sort/)\n\nThis package is a plugin that sort dependencies alphabetically in pyproject.toml\nafter running `poetry init`, `poetry add`, or `poetry remove`.\nSince [Introduce dependency sorting #3996](https://github.com/python-poetry/poetry/pull/3996) pull request still open\nthis plugin is a workaround for [!312](https://github.com/python-poetry/poetry/issues/312) issue.\n\n**Note**: the plugin is in the beta version!\n\n# Installation\n\nJust use `poetry self add` command to add this plugin.\n\n```bash\npoetry self add poetry-plugin-sort\n```\n\nIf you used pipx to install Poetry, add the plugin via `pipx inject` command.\n\n```bash\npipx inject poetry poetry-plugin-sort\n```\n\nAnd if you installed Poetry using pip, you can install the plugin the same way.\n\n```bash\npip install poetry poetry-plugin-sort\n```\n',
    'author': 'Andrei Shabanski',
    'author_email': 'shabanski.andrei@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/andrei-shabanski/poetry-plugin-sort',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from cleo.events import console_events
from cleo.events.console_command_event import ConsoleCommandEvent
from cleo.events.event_dispatcher import EventDispatcher
from cleo.io.io import IO
from poetry.console.application import Application
from poetry.console.commands.add import AddCommand
from poetry.console.commands.init import InitCommand
from poetry.console.commands.remove import RemoveCommand
from poetry.plugins.application_plugin import ApplicationPlugin

from poetry_plugin_sort.sort import Sorter


class SortDependenciesPlugin(ApplicationPlugin):
    """Sorts dependencies in pyproject.toml file"""

    def activate(self, application: Application):
        application.event_dispatcher.add_listener(
            console_events.TERMINATE, self.sort_dependencies
        )

    def sort_dependencies(
        self, event: ConsoleCommandEvent, event_name: str, dispatcher: EventDispatcher
    ) -> None:
        io = event.io
        command = event.command

        if event.exit_code != 0:
            self._write_debug_lines(
                io, "Skip sorting dependencies due to non-zero exit code."
            )
            return

        if not isinstance(command, (InitCommand, AddCommand, RemoveCommand)):
            self._write_debug_lines(
                io,
                f"Skip sorting dependencies due to {command} does not change the"
                " state.",
            )
            return

        if command.option("dry-run", False):
            self._write_debug_lines(
                io, "Skip sorting dependencies due to --dry-run option."
            )
            return

        sorter = Sorter(event.command.poetry, io)
        sorter.sort()

    def _write_debug_lines(self, io: IO, message: str) -> None:
        if io.is_debug():
            io.write_line(message)



from typing import Any
from typing import List
from typing import Tuple

from cleo.io.io import IO
from poetry.core.packages.dependency_group import MAIN_GROUP
from poetry.core.poetry import Poetry


class Sorter:
    def __init__(self, poetry: Poetry, io: IO):
        self._poetry = poetry
        self._io = io

    def sort(self):
        """Sorts dependencies from all groups and writes changes to pyproject.toml"""
        group_names = self._poetry.package.dependency_group_names(include_optional=True)
        for group in group_names:
            if group == MAIN_GROUP:
                dependency_toml_path = ["tool", "poetry", "dependencies"]
            else:
                dependency_toml_path = [
                    "tool",
                    "poetry",
                    "group",
                    group,
                    "dependencies",
                ]

            self._sort_dependencies_by_path(dependency_toml_path)

        # sort the legacy dev group
        self._sort_dependencies_by_path(["tool", "poetry", "dev-dependencies"])

        self._poetry.pyproject.save()
        self._io.write_line("Dependencies were sorted.")

    def _sort_dependencies_by_path(self, path: List[str]):
        dependency_section = _get_by_path(self._poetry.pyproject.data, path)
        if not dependency_section:
            return

        if self._io.is_debug():
            self._io.write_line(f'Sorting items in [{".".join(path)}].')

        items = tuple(dependency_section.items())
        for key, _ in items:
            dependency_section.remove(key)

        dependency_section.update(sorted(items, key=self._sort_key))

    def _sort_key(self, item: Tuple[str, Any]) -> str:
        package_name, package_version = item
        package_name = package_name.lower()

        # move python version to top
        if package_name == "python":
            package_name = f"____{package_name}"

        return package_name


def _get_by_path(d: dict, path: List[str]):
    """
    Gets a value from the dictionary by the path.
    """
    for key in path:
        d = d.get(key)
        if d is None:
            return None
    return d
